var jpath = require('json-path');
var fs = require('fs');
var _ = require('underscore');
var S = require('string');
var traverse = require('traverse');
require('buffertools').extend();
var utils = require('./utils');

//Note* Field and Property are interchangeable terms in this module

var CONSTANTS = {
	pushSides: {
		from: "from",
		to: "to"
	},
	limitTypes: {
		contains: "contains",
		equals: "equals",
		all: "all"
	}
};

//Push Class Constructor - Configurable in  a file
function Push(fromParent, fromTarget, fromTargetChildLimiter, fromTargetValueEquals, fromTargetValueContains,
		fromTargetRequired, toParent, toTarget, toTargetChildLimiter, toTargetValueEquals , toTargetValueContains, 
		toTargetRequired, toTargetExist, toParentId) {
	
	this.fromParent = fromParent;
	this.fromTarget = fromTarget;
	this.fromTargetChildLimiter = fromTargetChildLimiter;
	this.fromTargetValueEquals = fromTargetValueEquals;
	this.fromTargetValueContains = fromTargetValueContains;
	this.fromTargetRequired = fromTargetRequired;
  
	this.toParent = toParent;
	this.toTarget = toTarget;
	this.toTargetChildLimiter = toTargetChildLimiter;
	this.toTargetValueEquals = toTargetValueEquals;
	this.toTargetValueContains = toTargetValueContains;
	this.toTargetRequired = toTargetRequired;
	this.toTargetExist = toTargetExist;
	  
	this.toParentId = toParentId;
}
//Export the Push Class
module.exports.Push = Push;

// ToParent Class Constructor - Configurable in  a file 
// Will be used for sequencing of missing fields in "to" JSON
function ToParent(id, name, childrenSequence) {
	
	this.id = id;
	this.name = name;
	this.childrenSequence = childrenSequence;
}

//Export the Push Class
module.exports.ToParent = ToParent;

module.exports.validateJson2JsonPushConfiguration = 
	function validateJson2JsonPushConfiguration( pushConfig ) {
	
		var pushes = pushConfig.pushes;
		var toParents = pushConfig.toParents;
		var pushConfigurationInvalid = 'JSON Push Configuration Invalid - ';
		
		pushes.forEach(function(push) {
			
			if (S(push.fromParent).isEmpty()) {
				throw new Error(pushConfigurationInvalid + 
						'A "fromParent" property has not been defined for a push, it must be defined with at least "/" as the path');
			}
			
			if (S(push.fromTarget).isEmpty()) {
				throw new Error(pushConfigurationInvalid + 
						'A "fromTarget" property has not been defined for a push "fromParent" of' +
						' [' + push.fromParent + '].');
			}
			
			if (!S(push.fromTargetChildLimiter).isEmpty()) {
				
				if (S(push.fromTargetValueEquals).isEmpty() 
						&& S(push.fromTargetValueContains).isEmpty()) {
					throw new Error(pushConfigurationInvalid + 
							'A "fromTargetChildLimiter" property of [' + push.fromTargetChildLimiter
								+ '] has been defined for a push "fromTarget" of [' + push.fromTarget 
								+ ']. However, neither a ' + '"fromTargetValueEquals" nor a "fromTargetValueContains"'
								+ ' has been defined for the push.');
				}
			}
			
			if (S(push.toParent).isEmpty()) {
				throw new Error(pushConfigurationInvalid + 
						'A "toParent" property has not been defined for a push, it must be defined with at least "/" as the path');
			}
			
			if (S(push.toTarget).isEmpty()) {
				throw new Error(pushConfigurationInvalid + 
						'A "toTarget" property has not been defined for a push "toParent" of' +
						' [' + push.toParent + '].');
			}
			
			if (!S(push.toTargetChildLimiter).isEmpty()) {
				
				if (S(push.toTargetValueEquals).isEmpty() 
						&& S(push.toTargetValueContains).isEmpty()) {
					throw new Error(pushConfigurationInvalid + 
							'A "toTargetChildLimiter" property of [' + push.toTargetChildLimiter
								+ '] has been defined for a push "toTarget" of [' + push.toTarget 
								+ ']. However, neither a ' + '"toTargetValueEquals" nor a "toTargetValueContains"'
								+ ' has been defined for the push.');
				}
			}
			
			if (!_.isUndefined(push.toParentId)) {
				if (_.isUndefined(getToParent(push, toParents).id)) {
					throw new Error(pushConfigurationInvalid + 
							'A "toParentId" property of [' + push.toParentId
								+ '] has been defined for a push "toTarget" of [' + push.toTarget 
								+ ']. However, a matching parent "id" not was found in ' 
								+ 'the "toParents" configuration.');
				}
			}
			
		});
		
		toParents.forEach(function(toParent) {
			if (_.isUndefined(toParent.id)) {
				throw new Error(pushConfigurationInvalid + 
						'A "id" property has not been defined for a toParent.');
			}
			
			if (S(toParent.name).isEmpty()) {
				throw new Error(pushConfigurationInvalid + 
						'A "name" property has not been defined for a toParent "id" of' +
						' [' + toParent.id + '].');
			}
			
			//check if children sequence is an array
			if (!_.isUndefined(toParent.childrenSequence) 
					&& !(toParent.childrenSequence instanceof Array)) {			
				throw new Error(pushConfigurationInvalid + 
						'A "childrenSequence" property has been defined for a toParent "id" of' +
						' [' + toParent.id + ']. However, it is not a type of Array. If necessary, an empty' +
						' Array can be used for the "childrenSequence" in the push configuration as follows: ' + 
						'"childrenSequence": [{}] ');
			}
			
		});
		
		return true;
};

module.exports.validateExistenceOfRequiredTargets = 
	function validateExistenceOfRequiredTargets(json, pushConfig, side) {
	
		var pushes = pushConfig.pushes;
	
		var createErrorMessage = 
			function(path, field, limiter, equals, contains) {	
			    
				var errMsgBuffer = new Buffer('Required Target Not Present - ');
							
				errMsgBuffer = 
					errMsgBuffer.concat('A required push "Target" does not exist at path "');
				
				if (isPathAForwardSlash(path)) {
					errMsgBuffer = 
						errMsgBuffer.concat('/',field,'"');
				} else {
					errMsgBuffer = 
						errMsgBuffer.concat(path, '/',field,'"');				 
				}
					
				if (!S(limiter).isEmpty()) {					
					errMsgBuffer = errMsgBuffer.concat(', with a "TargetChildLimiter" of "', limiter,'" ');					
					if (!S(equals).isEmpty()) {
						errMsgBuffer = errMsgBuffer.concat('and a "TargetValueEquals" of "', equals, '"');
					}				
					if (!S(contains).isEmpty()) {
						errMsgBuffer = errMsgBuffer.concat('and a "TargetValueContains" of "', contains, '"');
					}
				}
				
				return errMsgBuffer;
			};
	
		pushes.forEach(function(push) {
			var resPushField;
			
			if (_.isEqual(side, CONSTANTS.pushSides.from)) {	
				if (!_.isUndefined(push.fromTargetRequired) 
						&& push.fromTargetRequired === true) {
					resPushField = pullJSONField (json, push, CONSTANTS.pushSides.from);
					
					if (_.isUndefined(resPushField)) {
						throw new Error(createErrorMessage(push.fromParent, 
								push.fromTarget, push.fromTargetChildLimiter,
								push.fromTargetValueEquals, push.fromTargetValueContains));
					}				
				}
			} else {
				//CONSTANTS.pushSides.to
				if (!_.isUndefined(push.toTargetRequired) 
						&& push.toTargetRequired === true) {
					resPushField = pullJSONField (json, push, CONSTANTS.pushSides.to);
					
					if (_.isUndefined(resPushField)) {
						throw new Error(createErrorMessage(push.toParent, 
								push.toTarget, push.toTargetChildLimiter,
								push.toTargetValueEquals, push.toTargetValueContains));
					}

				}
			}	
		});
		
		return true;
};

module.exports.initiate = function initiate(pushFromJSON, pushToJSON, pushConfig) {
	
	//Perform validation before allowing jsonPush to run
	try {
		module.exports.validateJson2JsonPushConfiguration(pushConfig);
		module.exports.validateExistenceOfRequiredTargets(pushFromJSON, pushConfig, CONSTANTS.pushSides.from);
		module.exports.validateExistenceOfRequiredTargets(pushToJSON, pushConfig, CONSTANTS.pushSides.to);
	} catch (err) {
		throw err;
	}
	
	var pushes = pushConfig.pushes;
	var toParents = pushConfig.toParents;
	
	return pushMultiple(pushFromJSON, pushToJSON, pushes, toParents);
};

//Function that will push json fields from one JSON to another, accepts either JSON Strings or JSON Objects
function pushMultiple(pushFromJSON, pushToJSON, pushes, toParents) {
	
	var pushToParsedJson;
	pushes.forEach(function(push) {
		if (pushToParsedJson) {
			pushToParsedJson = pushSingle(pushFromJSON, pushToParsedJson, push, toParents);
		} else {
			pushToParsedJson = pushSingle(pushFromJSON, pushToJSON, push, toParents);
		};		
	});
	
	return pushToParsedJson;	
};

function pushSingle(pushFromJSON, pushToJSON, push, toParents) {
	
	
	var resPushFromField = pullJSONField (pushFromJSON, push, CONSTANTS.pushSides.from);
	var resPushToField = pullJSONField (pushToJSON, push, CONSTANTS.pushSides.to);
	
	var mergedJSON = '{}';
	
	var pushToParsedJson = utils.parseJSON(pushToJSON);
	setToFieldExistPropertyOnPush(push, pushToParsedJson);
	
	if (push.toTargetExist === true 
			&&  isPathAForwardSlash(push.toParent)) {
		
		if (!_.isUndefined(resPushFromField && resPushFromField[push.fromTarget])) {
			resPushFromField = resPushFromField[push.fromTarget];	
		} else {
			resPushFromField = undefined;
		}
		
		if (!_.isUndefined(resPushToField && resPushToField[push.toTarget])) {
			resPushToField = resPushToField[push.toTarget];	
		} else {
			resPushToField = undefined;
		}

	}
	
	mergedJSON = mergeJSON(resPushFromField, resPushToField);	
		
	var toParent = getToParent(push, toParents);
	
	if (push.toTargetExist === true 				
			|| (push.toTargetExist === false 
					&& (!toParent.childrenSequence 
							|| toParent.childrenSequence.length < 1 ))) {		
		//if || OR part of the condition is reached and passes, it means the field doesn't exist on the target JSON
		//and it is not supported by a sequence, therefore it will be appended to the end of the given path
		var pushParentPath = createTraversePath(push.toParent, push.toTarget);
		
		if (push.toTargetExist === true 
				&&  isPathAForwardSlash(push.toParent) 
				&& mergedJSON.length === 1) {
			traverse(pushToParsedJson).set(pushParentPath, mergedJSON[0]);	
		} else {
			traverse(pushToParsedJson).set(pushParentPath, mergedJSON);	
		}
	} else {
		pushToParsedJson = rebuildTargetJSONUsingChildSequenceOrder(pushToParsedJson, push, toParent, mergedJSON);
	}
	
	return pushToParsedJson;	
};

function rebuildTargetJSONUsingChildSequenceOrder(pushToParsedJson, push, toParent, mergedJSON) {
	
	//1. Gets the correct order of children that are present
	var childrenSequenceOrder = getChildrenSequenceOrder(pushToParsedJson, push, toParent);

	//2. Adds new json into the correct property location according to the sequence provided.
	var newChildrenJSON = addNewJSONIntoChildrenJSON(pushToParsedJson, push, childrenSequenceOrder, mergedJSON);

	//3. Creates a parent path
	var pushParentPath = createTraversePath(push.toParent); // field was null

	//4. Sets the new reordered JSON into the correct location
	traverse(pushToParsedJson).set(pushParentPath, newChildrenJSON);
	
	return pushToParsedJson;
}

function getChildrenSequenceOrder(pushToParsedJson, push, toParent) {
	
	var pushParentPath = createTraversePath(push.toParent); // field was null

	var childrenArray = [];
	
	traverse(pushToParsedJson).forEach(
		function to_s(node) {				
			var nodeParentPath = [];
			if (this.parent && this.parent.path) {
				if (isPathAForwardSlash(push.toParent)) {
					nodeParentPath = [0];
				} else {
					nodeParentPath = this.parent.path;
				}

			} 				
			if (_.isEqual(pushParentPath, nodeParentPath)) {				
				var key = S(this.key).toString();
				if (key) {
					childrenArray.push(key);	
				}
			}									
		}	
	);
	
	childrenArray.push(push.toTarget);
	
	if (!_.contains(toParent.childrenSequence), push.toTarget) {
		toParent.childrenSequence.push(push.toTarget);
	}
	
	var sequenceOrder = _.intersection(toParent.childrenSequence, childrenArray);	

	return sequenceOrder;
}

function addNewJSONIntoChildrenJSON(pushToParsedJson, push, childrenSequenceOrder, mergedJSON) {
	
	var pushParentPath;	
	pushParentPath = createTraversePath(push.toParent);
	
	var element = traverse(pushToParsedJson).get(pushParentPath);

	var obj = new Object;	
	childrenSequenceOrder.forEach(function(seqOrder) { 					
		if (seqOrder == push.toTarget) {
			if (isPathAForwardSlash(push.toParent)) {
				if (!_.isEmpty(mergedJSON)) { 
					var jsStr;
					if (mergedJSON[0] && mergedJSON[0][push.fromTarget]) {
						jsStr = "obj[seqOrder] = mergedJSON[0]['" + push.fromTarget + "'];";
					} else {
						jsStr = "obj[seqOrder] = mergedJSON[0];";
					}
					eval(jsStr);		
				}
			} else {
				if ( mergedJSON && mergedJSON[0] && mergedJSON[0][push.fromTarget]) {
					var jsStr = "obj[seqOrder] = mergedJSON[0]['" + push.fromTarget + "'];";
					eval(jsStr);
				} else {
					if (mergedJSON && mergedJSON[0] && mergedJSON.length == 1) {
						obj[seqOrder] = mergedJSON[0];
					} else {
						obj[seqOrder] = mergedJSON;
					}
				}
			}
		} else {
			obj[seqOrder] = element[seqOrder];		
		}
	});
	
	return obj;
}

function pullJSONField (json, push, side) {
	
	var parsedJson = utils.parseJSON(json);
	var resPush = new Object;
	
	var pushPath;
	var pushField;
	var pushLimiter;
	var pushFieldEquals;
	var pushFieldContains;
	
	if (_.isEqual(side, CONSTANTS.pushSides.from)) {		
		pushPath = push.fromParent;
		pushField = push.fromTarget;
		pushLimiter = push.fromTargetChildLimiter;
		pushFieldEquals = push.fromTargetValueEquals;
		pushFieldContains = push.fromTargetValueContains;
		
	} else {
		//CONSTANTS.pushSides.to
		pushPath = push.toParent;
		pushField = push.toTarget;
		pushLimiter = push.toTargetChildLimiter;
		pushFieldEquals = push.toTargetValueEquals;
		pushFieldContains = push.toTargetValueContains;
	}
	
	var limitType = getLimitType(push, side);
	if (_.isEqual(limitType, CONSTANTS.limitTypes.equals)) {
		resPush = 
			limitResults(CONSTANTS.limitTypes.equals, pushPath, pushField, pushLimiter, pushFieldEquals, parsedJson); 
	} else if (_.isEqual(limitType, CONSTANTS.limitTypes.contains)) {
		resPush =				
			limitResults(CONSTANTS.limitTypes.contains, pushPath, pushField, pushLimiter, pushFieldContains, parsedJson); 
	} else {
		//all
		resPush = allResults(pushPath, pushField, parsedJson);
	}	
	
	if (isPathAForwardSlash(pushPath)) {
		return resPush[0];
	} else {
		if (resPush[0] && resPush[0][pushField]) {
			return resPush[0][pushField];
		} else {
			return undefined;
		}
	}
}

//field is optional
function createTraversePath(path, field) {
	
	if (isPathAForwardSlash(path)) {
		if (!_.isUndefined(field)) {
			path = field;
		} else {
			return path = [0];
		}
	} else {
		if (field) {
			path += '/' + field;
		}
	}	
	
	var pushParentPath = "[ '0'";
	var toParentArray = path.split("/");
	toParentArray.forEach(function(data) {
		pushParentPath += ",'" + data + "'";
	});
	pushParentPath += " ]";
	
	return eval(pushParentPath);
}

function isPathAForwardSlash(path) {
	return  _.isEqual(path, "/");
}

function mergeJSON(resPushFromField, resPushToField) {
	
	var mergedJSON;
	//Logic to handle or not handle arrays of the pushFrom data and push to Data.
	switch(true) {
		case (resPushFromField instanceof Array 
				&& !(resPushToField instanceof Array)):
			if (resPushFromField 
					&& resPushToField) {
				resPushFromField.unshift(resPushToField); //Add PushFrom to the Bottom
				mergedJSON = resPushFromField;
			} else if (resPushFromField 
						&& !resPushToField) {
				mergedJSON = resPushFromField;
			} else if (!resPushFromField 
					&& resPushToField) {
				mergedJSON = resPushToField;
			} else {
				mergedJSON = new Array();
			}

			break;
		case (resPushToField instanceof Array 
				&& !(resPushFromField instanceof Array)):
			if (!_.isUndefined(resPushFromField)) {
				resPushToField.push(resPushFromField);
			}
			mergedJSON = resPushToField;

			break;
		case (resPushToField instanceof Array 
				&& resPushFromField instanceof Array):
					var concatenated = resPushToField.concat(resPushFromField);
			mergedJSON = concatenated;

			break;
		case (!(resPushToField instanceof Array) 
				&& !(resPushFromField instanceof Array)):
			var ncDocIdArray = new Array();
			if (resPushToField) {
				ncDocIdArray.push(resPushToField);
			}
			if (resPushFromField) {
				ncDocIdArray.push(resPushFromField);
			}						
			mergedJSON = ncDocIdArray;

			break;
		default:
			break;
	}
	
	return mergedJSON;
}

function getLimitType(push, pushSide) {
	
	if ( _.isEqual(pushSide, CONSTANTS.pushSides.from)) {
		//from
		return getLimitTypeGeneric(push.fromTargetChildLimiter, push.fromTargetValueContains, push.fromTargetValueEquals);
	} else if (_.isEqual(pushSide, CONSTANTS.pushSides.to)) {
		//to
		return getLimitTypeGeneric(push.toTargetChildLimiter, push.toTargetValueContains, push.toTargetValueEquals);
	} else {
		//throw error here, internal application, call did not set pushSide correctly
	}
}

function getLimitTypeGeneric(fieldLimiter, fieldContains, fieldEquals) {
	
	var limitType;
	if (!S(fieldLimiter).isEmpty()
			&& !S(fieldContains).isEmpty()) {
		limitType = CONSTANTS.limitTypes.contains;
	} else if (!S(fieldLimiter).isEmpty()
			&& !S(fieldEquals).isEmpty()) {
		limitType = CONSTANTS.limitTypes.equals;
	} else {
		limitType = CONSTANTS.limitTypes.all;
	}
	
	return limitType;
}

function allResults(pushPath, pushField, parsedJson) {

	var psStr;
	
	if (isPathAForwardSlash(pushPath)) { 
		
		var ps = jpath.parseSelector('/' + pushField);
		var exist = jpath.executeSelectors(parsedJson, ps);
		
		if (exist.length > 0) {
			psStr = 'take(/' + pushField + ')';
		} else {
			psStr = '/' + pushField;
		}
				
	} else {
		psStr = '#/' + pushPath + '[take(/' + pushField + ')]';
	}
	
	var psPush = jpath.parseSelector(psStr);
	var resPush = jpath.executeSelectors(parsedJson, psPush);

	return resPush;
}

function limitResults(limitType, pushPath, pushField, pushFieldLimiter, pushLimiter, parsedJson) {
	
	if (_.isEqual(limitType.toLowerCase(), CONSTANTS.limitTypes.equals)) {
		return limitResultsEqualIgnoreCase(pushPath, pushField, pushFieldLimiter, pushLimiter, parsedJson);		
	} else {
		//contains
		return limitResultsContainsIgnoreCase(pushPath, pushField, pushFieldLimiter, pushLimiter, parsedJson);
	}
}

function formatJSONResponse(pushPath, pushField,resPush) {
	
	if (!_.isUndefined(resPush[0]) 
			&& !isPathAForwardSlash(pushPath) ) {
		return JSON.parse('[ { "' + pushField + '" :' + JSON.stringify(resPush[0])  +' }]');
	} else {
		return JSON.parse('[ { "' + pushField + '" :' + JSON.stringify(resPush)  +' }]');
	}
}

function limitResultsEqualIgnoreCase(pushPath, pushField, pushFieldLimiter, pushLimiter, parsedJson) {
	
	var psStr;
	if (isPathAForwardSlash(pushPath)) { 
		psStr = '[#/' + pushField + '[*][@fn]]';			
	} else {
		psStr = '[#/' + pushPath + '/' + pushField + '[*][@fn]]';
	}
	
	var psPush = jpath.parseSelector(psStr);
	var resPush = jpath.executeSelectors(parsedJson, psPush, {
		fn : function(obj, accum, sel)  {	
			if (isPathAForwardSlash(pushPath)) { 
				if (obj				
						&& _.isEqual(S(obj).toString().toLowerCase(), 
								pushLimiter.toLowerCase())) {				
					accum.push(obj);
				}
			} else {
				if (obj[pushFieldLimiter] 				
				&& _.isEqual(S(obj[pushFieldLimiter]).toString().toLowerCase(), 
						pushLimiter.toLowerCase())) {				
			accum.push(obj);
		}
			}
			return accum;
		}
	});

	if (!_.isUndefined(resPush) 
			&& resPush.length > 0) {
		resPush = formatJSONResponse(pushPath, pushField, resPush);
	} else {
		//else checks for non array to work with	
		var path = createTraversePath(pushPath, pushField);
		var obj = traverse(parsedJson).get(path);
		
		if (!_.isUndefined(obj)) {	
			
			if (isPathAForwardSlash(pushPath)) { 
				
				if (_.isEqual(S(obj).toString().toLowerCase(), 
						pushLimiter.toLowerCase())) {
					resPush = formatJSONResponse(pushPath, pushField, obj);
				} else {	
					resPush = [];
				}
				
			} else {
			
				if (obj[pushFieldLimiter] 				
						&& _.isEqual(S(obj[pushFieldLimiter]).toString().toLowerCase(), 
								pushLimiter.toLowerCase())) {
					resPush = formatJSONResponse(pushPath, pushField, obj);
				} else {	
					resPush = [];
				}
			}
			
		} else {	
			resPush = [];
		}
	}
	
	return resPush;
}

function limitResultsContainsIgnoreCase(pushPath, pushField, pushFieldLimiter, pushLimiter, parsedJson) {
	
	var psStr;
	if (isPathAForwardSlash(pushPath)) { 
		psStr = '[#/' + pushField + '[*][@fn]]';			
	} else {
		psStr = '[#/' + pushPath + '/' + pushField + '[*][@fn]]';
	}
	
	var psPush = jpath.parseSelector(psStr);
	var resPush = jpath.executeSelectors(parsedJson, psPush, {
		fn : function(obj, accum, sel)  {
			if (isPathAForwardSlash(pushPath)) { 
				if(obj &&  
						S(obj).toString().toLowerCase().indexOf(pushLimiter.toLowerCase()) > -1) {
					accum.push(obj);
				}
			} else {
				if(obj[pushFieldLimiter] &&  
						S(obj[pushFieldLimiter]).toString().toLowerCase().indexOf(pushLimiter.toLowerCase()) > -1) {
					accum.push(obj);
				}	
			}
			return accum;
		}
	});
	
	if (!_.isUndefined(resPush) 
			&& resPush.length > 0) {
		resPush = formatJSONResponse(pushPath, pushField, resPush);
	} else {
		//else checks for non array to work with		
		var path = createTraversePath(pushPath, pushField);
		var obj = traverse(parsedJson).get(path);
		
		if (!_.isUndefined(obj)) {	
			
			if (isPathAForwardSlash(pushPath)) { 
				if(S(obj).toString().toLowerCase().indexOf(pushLimiter.toLowerCase()) > -1) {
					resPush = formatJSONResponse(pushPath, pushField, obj);
				} else {	
					resPush = [];
				}				
			} else {
				if(obj[pushFieldLimiter] &&  
						S(obj[pushFieldLimiter]).toString().toLowerCase().indexOf(pushLimiter.toLowerCase()) > -1) {
					resPush = formatJSONResponse(pushPath, pushField, obj);
				} else {	
					resPush = [];
				}
			}
			
		} else {	
			resPush = [];
		}
	}
	
	return resPush;
}

function getToParent(push, toParents) {	
	
	var psPush = jpath.parseSelector('#[@fn]');
	var resPush = jpath.executeSelectors(toParents, psPush, {
		fn : function(obj, accum, sel)  {
			if (obj.id && obj.id === push.toParentId)
			    accum.push(obj);
			  return accum;
		}
	});
	
	if (!_.isUndefined(resPush) 
			&& resPush.length > 0) {
		resPush = resPush[0];
	} else {
		resPush = {};
	}
	
	return resPush;
}

function setToFieldExistPropertyOnPush(push, pushToParsedJson) { 
	
	var path = createTraversePath(push.toParent, push.toTarget); 
	var fieldExist = traverse(pushToParsedJson).get(path);
	
	if (fieldExist) {
		push.toTargetExist = true;
	} else {
		push.toTargetExist = false;
	}	
}