/**
 * getSsn.js - module to get ssn FROM DBQ Metadata submitted
 * 
 * author: Barb Hals
 * 
 */
"use strict";

var path = require('path');
var fs = require('fs');
var jpath = require('json-path');
var _ = require('underscore');
var logger = module.parent.exports.logger;

/*
 * Process options paths
 */

function performOperation(resolvedPathsObject, element) {
    if (element.operation == "indexOf") {
        logger.debug(path.basename(__filename) + ": " + "Matching Index.");
        var indexOfArray = resolvedPathsObject[element.array];
        var indexOfValue = resolvedPathsObject[element.value];
        logger.debug(path.basename(__filename) + ": " + indexOfArray, element.value, indexOfValue);
        var operationResult = _.indexOf(indexOfArray, indexOfValue);
        logger.debug(path.basename(__filename) + ": " + "Operation result: " + operationResult);

        resolvedPathsObject[element.saveVariable] = operationResult;
        logger.debug(path.basename(__filename) + ": " + "resolvedPathsObject:  " + JSON.stringify(resolvedPathsObject));
    }

}

function logHelperForUnknownVariableType(resolvedPath) {
    if (_.isArray(resolvedPath))
        logger.debug(path.basename(__filename) + ": " + "resolvedPath (array-before applying options index): "
                + JSON.stringify(resolvedPath));
    else if (_.isObject(resolvedPath))
        logger.debug(path.basename(__filename) + ": " + "resolvedPath (object): " + JSON.stringify(resolvedPath));
    else
        logger.debug(path.basename(__filename) + ": " + "resolvedPath:  " + resolvedPath);
}

// http://stackoverflow.com/questions/22643038/split-string-and-keep-the-deliminter
function processVariableReplacement(element, resolvedPathsObject) {
    var res = element.path.replace(/\$\{([^}]+)}/g, function(match, gr) {
        return gr in resolvedPathsObject ? resolvedPathsObject[gr] : match;
    });

    logger.debug(path.basename(__filename) + ": " + "Result after Variable Replacement: " + res);

    return res;
}

// This is the underscore iterator in _.map(element, index, list)
// The resolvedPathsParentObject passed as the context (this)
function processEachJSONPathObject(element, index, list) {
    // Log the JSONPathObject
    logger.debug(path.basename(__filename) + ": *JSONPathObject:" + JSON.stringify(element));

    // We need to perform an operation
    if (element.operation) {
        logger.debug(JSON.stringify(this));
        performOperation(this["resolvedPathsObject"], element);
        return;
    }

    // Handle Variable Replacements
    element.path = processVariableReplacement(element, this["resolvedPathsObject"]);
    logger.debug(path.basename(__filename) + ": " + "Path after variable replacement (if any): "
            + JSON.stringify(element));
    var resolvedPath = jpath.resolve(this.sourceFile, element.path);

    // Handle Array Index from the Configuration File (if applicable)
    var arrayIndex = null;
    if (element.arrayIndex) {
        arrayIndex = parseInt(element.arrayIndex);
    }
    logger.debug(path.basename(__filename) + ": " + "Configuration array index: " + arrayIndex);
    if (!_.isNaN(arrayIndex) && !_.isNull(arrayIndex)) {
        resolvedPath = resolvedPath[arrayIndex];
    }

    // Logging
    logger.debug(path.basename(__filename) + ": " + "resolvedPath type: " + typeof resolvedPath);
    logHelperForUnknownVariableType(resolvedPath);

    // Now update the resolvedPathsObject
    logger.debug(path.basename(__filename) + ": " + "Saving to element.saveVariable: " + element.saveVariable);
    this["resolvedPathsObject"][element.saveVariable] = resolvedPath;
    logger.debug(path.basename(__filename) + ": " + "element.returnVar : " + this["resolvedPathsObject"][element.returnVar]);

    // Check if we're ready to return
    if (element.returnVar) {
        // Stored in last mapresult value
        return this["resolvedPathsObject"][element.returnVar];
    }
}

/*
 * Loop through each of the JSONPathObjects in the optionsuration.
 * resolvedPathsObject maintains all of the resolved paths from the
 * optionsuration in variables/object properties using the variable names
 * specified in the optionsuration.
 * 
 * throws Error if any error is found in parsing the options or
 */
function resolvePaths(sourceFile, options) {
    try {
        var resolvedPathsParentObject = {};
        resolvedPathsParentObject.sourceFile = sourceFile;

        resolvedPathsParentObject["resolvedPathsObject"] = {};

        // Process each Path. resolvedPathsObject is passed as the
        // context (this in the iterator) _.map(list, iterator, [context])
        // list is an array in this case
        var results = _.map(options, processEachJSONPathObject, resolvedPathsParentObject);

        logger.info(path.basename(__filename) + ": " + "Result resolvePaths: " + results[options.length - 1]);
        return results[options.length - 1];

    } catch (err) {
        logger.error(path.basename(__filename) + ": " + "Error Processing Path: " + err.message);
        logger.error(new Error().stack);
        throw err; // Let caller handle error
    }
}

module.exports = resolvePaths;
