/**
 * getSSNfromCommonData.js - module to test getSsn.js functionality
 * 
 * This mocha file needs to be run from the command line. It won't run in
 * Eclipse. First install mocha globally: npm install -g mocha
 * 
 * Run all tests in 'test' directory from root project directory: shell> mocha
 * 
 * No need to change the Makefile author: Barb Hals
 */
"use strict";

var config = require('config');
// require('mocha');

// Logging - overrides default config
// Override these values in default config)
config.log.file.silent = true; // Turn off file logging
config.log.console.level = 'debug'; // Don't show full DBQ JSON

function executeTest(beforeDescription, testDescription, filename, thisConfig, expectedResult) {
    // read input file for test
    /*
     * var source_file = fs.readFileSync(__dirname + '/' + filename, 'utf8');
     * console.log(source_file + '\n'); var parsedDbq = JSON.parse(source_file,
     * null); var clientSsn = resolvePaths.getSSNfromCommonData(parsedDbq);
     */

    describe(beforeDescription, function() {
        var clientSsn = null;

        before(function(done) {
            var source_file = fs.readFileSync(__dirname + '/' + testResourcesFolder + "/" + filename, 'utf8');
            // console.log(source_file + '\n');
            var parsedDbq = JSON.parse(source_file, null);
            clientSsn = resolvePaths(parsedDbq, thisConfig);
            done();
        });

        // Test
        it(testDescription, function() {
            logger.debug("returned value type: " + typeof clientSsn + " value: " + clientSsn);
            expect(clientSsn).to.contain(expectedResult);
            // user.should.have.property('username','test');
        });
    });
}

var logger = require('vcommons').log.getLogger('vlerdas-http-enricher', config.log);
module.exports.logger = logger;

var expect = require('expect.js');
var fs = require('fs');
var _ = require('underscore');
var resolvePaths = require('../lib/resolvePaths');

var beforeDescription = "Retrieve the SSN";
var testDescription = 'should have the correct SSN';

var testResourcesFolder = "resolvePaths";

// Default Configuration for these
executeTest(beforeDescription + " (DBQ Metadata):", testDescription, 'DBQ_Metadata.json', config.resolvePaths,
        "999428888");
executeTest(beforeDescription + " (Generic NIEM Metadata):", testDescription, 'Generic_NIEM_file.json',
        config.resolvePaths, "999428888");

// Use separate configuration
var thisConfigText = fs.readFileSync(__dirname + '/' + testResourcesFolder + "/" + "simpleJsonPathConfig.json", 'utf8');
var thisConfig = JSON.parse(thisConfigText, null);
executeTest(beforeDescription + " (Simple JSON Path):", testDescription, 'simpleJsonPath.json',
        thisConfig.resolvePaths, "zasdf");

//TO DO: Multiple variables test